@@ -63,7 +63,7 @@ rdrobust = function(y, x, c = NULL, fuzzy = NULL, deriv = NULL,
 
   if (!is.null(weights)){
     if (!is.null(subset)) weights <- weights[subset]
-    na.ok <- na.ok & complete.cases(weights)
+    na.ok <- na.ok & complete.cases(weights) & weights>=0
   } 
   
   x = as.matrix(x[na.ok])
